/*
    DefeoB - a bughouse chess playing program
    Copyright (C) 2010 SKAcz.
	  Originally based on Sjeng85 Copyright (C) 2000-2001 Gian-Carlo Pascutto

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    File: attacks.c                                             
    Purpose: calculate attack information                      
 
*/

#include "DefeoB.h"
#include "extvars.h"

int calc_attackers (int square, int color) 
{

  /* this function calculates attack information for a square */

  static const int rook_o[4] = {12, -12, 1, -1};
  static const int bishop_o[4] = {11, -11, 13, -13};
  static const int knight_o[8] = {10, -10, 14, -14, 23, -23, 25, -25};
  int a_sq, i;

  int attackers = 0;

  if (board[square] == frame) return 0;

  /* white attacker: */
  if (color&1) 
  {
    /* rook-style moves: */
    for (i = 0; i < 4; i++) 
	{
      a_sq = square + rook_o[i];

      /* the king can attack from one square away: */
      if (board[a_sq] == wking) 
	  {	 attackers++;
	     break;
	  }
      else
	  {
	    /* otherwise, check for sliding pieces: */
	    while (board[a_sq] != frame) 
		{
	      if (board[a_sq] == wrook || board[a_sq] == wqueen) 
	      {  attackers++;
		     break;
	      }
	      else if (board[a_sq] != npiece) break;
	         a_sq += rook_o [i];
		}
	  }
	}

    /* bishop-style moves: */
    for (i = 0; i < 4; i++) 
	{
      a_sq = square + bishop_o[i];
      /* check for pawn attacks: */
      if (board[a_sq] == wpawn && i%2)
	  {  attackers++;
	     break;
	  }
        /* the king can attack from one square away: */
      else if (board[a_sq] == wking)
	  {  attackers++;
	     break;
	  }
      else
	  {
	    while (board[a_sq] != frame) 
		{
	     if (board[a_sq] == wbishop || board[a_sq] == wqueen) 
		 {  attackers++;
		    break;
		 }
	     else if (board[a_sq] != npiece) break;
	     a_sq += bishop_o [i];
		}
	  }
	}

    /* knight-style moves: */
    for (i = 0; i < 8; i++) 
	{
      a_sq = square + knight_o[i];
      if (board[a_sq] == wknight)
	  attackers++;
	  
    }

    /* if we haven't hit a white attacker by now, there are none: */

  }

  /* black attacker: */
  else 
  {
    /* rook-style moves: */
    for (i = 0; i < 4; i++) 
	{
      a_sq = square + rook_o[i];
      /* the king can attack from one square away: */
      if (board[a_sq] == bking)
	  {  attackers++;
	     break;
	  }
      /* otherwise, check for sliding pieces: */
      else 
	  {
	    while (board[a_sq] != frame) 
		{
	      if (board[a_sq] == brook || board[a_sq] == bqueen) 
		  {  attackers++;
	         break;
		  };
	      if (board[a_sq] != npiece) break;
	      a_sq += rook_o [i];
		}
	  }
	}

    /* bishop-style moves: */
    for (i = 0; i < 4; i++) 
	{
      a_sq = square + bishop_o[i];
      /* check for pawn attacks: */
      if (board[a_sq] == bpawn && !(i%2))
	  {
	    attackers++;
	    break;
	  }
      /* the king can attack from one square away: */
      else if (board[a_sq] == bking)
	  {
	    attackers++;
	    break;
	  }
      else 
	  {
	    while (board[a_sq] != frame) 
		{
	      if (board[a_sq] == bbishop || board[a_sq] == bqueen) 
		  { attackers++;
	        break;
		  }
	      else if (board[a_sq] != npiece) break;
	      a_sq += bishop_o [i];
		}
	  }
	}

    /* knight-style moves: */
    for (i = 0; i < 8; i++) 
	{
      a_sq = square + knight_o[i];
      if (board[a_sq] == bknight) 
	    attackers++;
    }
    /* if we haven't hit a black attacker by now, there are none: */
  }
  return attackers;
}


void calc_energy2_board()
{
  // calculates energy of pieces but only related to kings defense
  // in centi units, so 0.5 means 50
  //  so it saids how big probability is that pieces defending some square
  // ie. if Bishop always defending squares on 1 diagonal with 0.5
  // difference between B and P is that Bg2xh3xg2 is possible, but Pg2xh3 only and no back
  // Q defending many close squares most like 7/9
  // the rest like R
  // but it has to work that 3 pieces (knights or B) every defending 2 squares
  //     from 4 means that suma on 4 squares will be 3=0.5+1+1+0.5
  //     what shows weakes point in defense after drop on which
  //     pieces will move and one from squares will stay undefended at the end
  //     thats 3energy on 4squares = -1
  //


}



void calc_energy_board()
{
  // this function calculates attack Energy for all squares of board //
  // calc cumulated attack for a square    //
  //  e.g. in line wQ|-|wB|wB|-|wP|Square| as 4 //
  //               wQ|-|bB|wB|-|wP|Square| as 2 //
  //
  //   

  // Out:
  //  board_energy[144]
  //  return Energy >0 for White superiority; <0 for Black


  //Function Version 2003112001,20081208

  static const int rook_o[4] = {12, -12, 1, -1};
  static const int bishop_o[4] = {11, -11, 13, -13};
  static const int knight_o[8] = {10, -10, 14, -14, 23, -23, 25, -25};

  int init_board_zero[144] = 
  {
  0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0
  };

  int a_sq0,a_sq,i,j;
  int energy,energy_w,energy_b, en_d;
  int board_tied[144];


  // set energy board to zero
     energy=0;energy_w=0;energy_b=0;
     memcpy (board_e_w, init_board_zero, sizeof (init_board_zero));
     memcpy (board_e_b, init_board_zero, sizeof (init_board_zero));
     memcpy (board_tied, init_board_zero, sizeof (init_board_zero));


     
     //printf ("!D0!49E:%d!49P:%d!E:%d!",board_energy[49],board[49],energy);


  //At first get tied pieces
  // Tied pieces can attacks max in 2 directions of in tied line,
  // (e.g. bR--WR--WK, so WR has just 2 free directions
  //  e.g. WB-bN-bK, so bN has no free directions, no moves)
  // (exist max 8 tied pieces for white, max 8 for black)

  if (wking_loc)
  {//white tied
     // bishop-style moves: //
      for (i = 0; i < 4; i++)
      {
        j=0;
        a_sq = wking_loc+bishop_o[i];
        while (board[a_sq] != frame && j != 1)
        {
	  if (board[a_sq]!=npiece)
          {
            if (board[a_sq]&1) //if white piece founded
            {
              if (j==0) j=a_sq;
              else j=1; //2 white pieces on line so no tied//
            }
            else //black
            {
              if (j && board[a_sq]==bbishop || board[a_sq]==bqueen)
              {
                //lower ability for tied white piece//
                board_tied[j]=abs(bishop_o[i]);
              }
              j=1;
            }
          }
          a_sq=a_sq+bishop_o[i];
        }
      }
     // rook-style moves: //
      for (i = 0; i < 4; i++)
      {
        j=0;
        a_sq = wking_loc+rook_o[i];
        while (board[a_sq] != frame && j != 1)
        {
	  if (board[a_sq]!=npiece)
          {
            if (board[a_sq]&1) // if white piece founded
            {
              if (j==0) j=a_sq;
              else j=1; //2 white pieces on line so no tied//
            }
            else //black
            {
              if (j && board[a_sq]==brook || board[a_sq]==bqueen)
              {
                //lower ability for tied white piece//
                board_tied[j]=abs(rook_o[i]);
              }
              j=1;
            }
          }
          a_sq=a_sq+rook_o[i];
        }
      }
  };
  if (bking_loc)
  {//black tied
     // bishop-style moves: //
      for (i = 0; i < 4; i++)
      {
        j=0;
        a_sq = bking_loc+bishop_o[i];
        while (board[a_sq] != frame && j != 1)
        {
	  if (board[a_sq]!=npiece)
          {
            if (!(board[a_sq]&1)) //if black piece founded
            {
              if (j==0) j=a_sq;
              else j=1; //2 black pieces on line so no tied//
            }
            else //white
            {
              if (j && board[a_sq]==wbishop || board[a_sq]==wqueen)
              {
                //lower ability for tied black piece//
                board_tied[j]=abs(bishop_o[i]);
              }
              j=1;
            }
          }
          a_sq=a_sq+bishop_o[i];
        }
      }
     // rook-style moves: //
      for (i = 0; i < 4; i++)
      {
        j=0;
        a_sq = bking_loc+rook_o[i];
        while (board[a_sq] != frame && j != 1)
        {
	  if (board[a_sq]!=npiece)
          {
            if (!(board[a_sq]&1)) //If black piece founded
            {
              if (j==0) j=a_sq;
              else j=1; //2 black pieces on line so no tied//
            }
            else //white
            {
              if (j && board[a_sq]==wrook || board[a_sq]==wqueen)
              {
                //lower ability for tied black piece//
                board_tied[j]=abs(rook_o[i]);
              }
              j=1;
            }
          }
          a_sq=a_sq+rook_o[i];
        }
      }
  };

  //Calc Energy emit by pieces on board
  // as Energy = WhiteSum - BlackSum of attackers for each square

  // loop through the board for all pieces encountered: //
  for (a_sq0 = 26;(a_sq0 < 118); a_sq0++)
  {
    if (board[a_sq0]==npiece) continue;


    if (!(board[a_sq0]&1)) //Black piece
    {board_e_b[a_sq0]++;}
    else //White piece
     {board_e_w[a_sq0]++;};

    switch (board[a_sq0])
    {
     case (wpawn):
       if (board_tied[a_sq0])
       {
         board_e_w[a_sq0+board_tied[a_sq0]]++;
       }
       else
       {
         board_e_w[a_sq0+11]++;
         board_e_w[a_sq0+13]++;
       }
     break;
     case (bpawn):
       if (board_tied[a_sq0])
       {
         board_e_b[a_sq0-board_tied[a_sq0]]++;
       }
       else
       {
         board_e_b[a_sq0-11]++;
         board_e_b[a_sq0-13]++;
       }
     break;
     case (wbishop):
      // bishop-style moves: //
      for (i = 0; i < 4; i++)
      {
        if (board_tied[a_sq0])
        {
          if (abs(bishop_o[i])!=board_tied[a_sq0]) continue;
        }
        a_sq = a_sq0 + bishop_o[i];
        while (board[a_sq] != frame)
        {
          board_e_w[a_sq]++;
	  if ((!board_tied[a_sq] || board_tied[a_sq]==abs(bishop_o[i]))
              && ( board[a_sq]==npiece || board[a_sq]==wqueen || board[a_sq]==wbishop)
             )
             a_sq += bishop_o[i];
          else
          if ((!board_tied[a_sq] || board_tied[a_sq]==abs(bishop_o[i]))
              &&  (board[a_sq]==wpawn && bishop_o[i]>0)
             )
          {
           a_sq += bishop_o[i];
           board_e_w[a_sq]++;
           break;
          }
          else break;
        }
      }
     break;
     case (bbishop):
      // bishop-style moves: //
      for (i = 0; i < 4; i++)
      {
        if (board_tied[a_sq0])
        {
          if (abs(bishop_o[i])!=board_tied[a_sq0]) continue;
        }
        a_sq = a_sq0 + bishop_o[i];
        while (board[a_sq] != frame)
        {
          board_e_b[a_sq]++;
	  if ((!board_tied[a_sq] || board_tied[a_sq]==abs(bishop_o[i]))
              && ( board[a_sq]==npiece || board[a_sq]==bqueen || board[a_sq]==bbishop )
             )
           a_sq += bishop_o[i];
          else
          if ((!board_tied[a_sq] || board_tied[a_sq]==abs(bishop_o[i]))
              && (board[a_sq]==bpawn && bishop_o[i]<0)
             )
          {
           a_sq += bishop_o[i];
           board_e_b[a_sq]++;
           break;
          }
          else break;
        }
      }
     break;
     case (wknight):
      // knight-style moves: //
      if (board_tied[a_sq0]==0)
       for (i = 0; i < 8; i++)
       {
         a_sq = a_sq0 + knight_o[i];
         board_e_w[a_sq]++;
       }
     break;
     case (bknight):
      // knight-style moves: //
       if (board_tied[a_sq0]==0)
        for (i = 0; i < 8; i++)
        {
          a_sq = a_sq0 + knight_o[i];
          board_e_b[a_sq]++;
        }
     break;
     case (wrook):
      // rook-style moves: //
      for (i = 0; i < 4; i++)
      {
        if (board_tied[a_sq0])
        {
          if (abs(rook_o[i])!=board_tied[a_sq0]) continue;
        }
        a_sq = a_sq0 + rook_o[i];
        while (board[a_sq] != frame)
        {
          board_e_w[a_sq]++;
	  if ( (!board_tied[a_sq] || board_tied[a_sq]==abs(rook_o[i]))
              && ( board[a_sq]==npiece || board[a_sq]==wqueen || board[a_sq]==wrook)
             )
             a_sq += rook_o[i];
	  else break;
        }
      }
     break;
     case (brook):
      // rook-style moves: //
      for (i = 0; i < 4; i++)
      {
        if (board_tied[a_sq0])
        {
          if (abs(rook_o[i])!=board_tied[a_sq0]) continue;
        }
        a_sq = a_sq0 + rook_o[i];
        while (board[a_sq] != frame)
        {
          board_e_b[a_sq]++;
	  if ((!board_tied[a_sq] || board_tied[a_sq]==abs(rook_o[i]))
              && ( board[a_sq]==npiece || board[a_sq]==bqueen || board[a_sq]==brook)
             )
            a_sq += rook_o[i];
	  else break;
        }
      }
     break;
     case (wqueen):
      // bishop-style moves: //
      for (i = 0; i < 4; i++)
      {
        if (board_tied[a_sq0])
        {
          if (abs(bishop_o[i])!=board_tied[a_sq0]) continue;
        }
        a_sq = a_sq0 + bishop_o[i];
        while (board[a_sq] != frame)
        {
          board_e_w[a_sq]++;
	  if ((!board_tied[a_sq] || board_tied[a_sq]==abs(bishop_o[i]))
              && ( board[a_sq]==npiece || board[a_sq]==wqueen || board[a_sq]==wbishop)
              )
           a_sq += bishop_o[i];
          else
          if ((!board_tied[a_sq] || board_tied[a_sq]==abs(bishop_o[i]))
              && (board[a_sq]==wpawn && bishop_o[i]>0)
             )
          {
           a_sq += bishop_o[i];
           board_e_w[a_sq]++;
           break;
          }
          else break;
        }
      }
      // rook-style moves: //
      for (i = 0; i < 4; i++)
      {
        if (board_tied[a_sq0])
        {
          if (abs(rook_o[i])!=board_tied[a_sq0]) continue;
        }
        a_sq = a_sq0 + rook_o[i];
        while (board[a_sq] != frame)
        {
          board_e_w[a_sq]++;
	  if ((!board_tied[a_sq] || board_tied[a_sq]==abs(rook_o[i]))
              && ( board[a_sq]==npiece || board[a_sq]==wqueen || board[a_sq]==wrook)
             )
          a_sq += rook_o [i];
	  else break;
        }
      }
     break;
     case (bqueen):
      // bishop-style moves: //
      for (i = 0; i < 4; i++)
      {
        if (board_tied[a_sq0])
        {
          if (abs(bishop_o[i])!=board_tied[a_sq0]) continue;
        }
        a_sq = a_sq0 + bishop_o[i];
        while (board[a_sq] != frame)
        {
          board_e_b[a_sq]++;
	  if ((!board_tied[a_sq] || board_tied[a_sq]==abs(bishop_o[i]))
              && ( board[a_sq]==npiece || board[a_sq]==bqueen || board[a_sq]==bbishop )
             )
           a_sq += bishop_o[i];
          else
          if ((!board_tied[a_sq] || board_tied[a_sq]==abs(bishop_o[i]))
              && (board[a_sq]==bpawn && bishop_o[i]<0)
             )
          {
           a_sq += bishop_o[i];
           board_e_b[a_sq]++;
           break;
          }
          else break;
        }
      }
      // rook-style moves: //
      for (i = 0; i < 4; i++)
      {
        if (board_tied[a_sq0])
        {
          if (abs(rook_o[i])!=board_tied[a_sq0]) continue;
        }
        a_sq = a_sq0 + rook_o[i];
        while (board[a_sq] != frame)
        {
          board_e_b[a_sq]++;
	  if ((!board_tied[a_sq] || board_tied[a_sq]==abs(rook_o[i]))
              && ( board[a_sq]==npiece || board[a_sq]==bqueen || board[a_sq]==brook )
             )
          a_sq += rook_o [i];
	  else break;
        }
      }
     break;
     case (wking):
     break;
     case (bking):
     break;

    }
  }

      board_e_w[wking_loc]--;
      board_e_b[bking_loc]--;

  //board_energy = board_e_w - board_e_b
    for (i = 26; i < 34; i++)
    { //if pieces on square defended by atleast one piece +1 energy for pieces color
      board_energy[i]=board_e_w[i] -board_e_b[i];
    }
    for (i = 38;i < 46; i++)
    { //if pieces on square defended by atleast one piece +1 energy for pieces color
      board_energy[i]=board_e_w[i] -board_e_b[i];
    }
    for (i = 50;i < 58; i++)
    { //if pieces on square defended by atleast one piece +1 energy for pieces color
      board_energy[i]=board_e_w[i] -board_e_b[i];
    }
    for (i = 62;i < 70; i++)
    { //if pieces on square defended by atleast one piece +1 energy for pieces color
      board_energy[i]=board_e_w[i] -board_e_b[i];
    }
    for (i = 74;i < 82; i++)
    { //if pieces on square defended by atleast one piece +1 energy for pieces color
      board_energy[i]=board_e_w[i] -board_e_b[i];
    }
    for (i = 86;i < 94; i++)
    { //if pieces on square defended by atleast one piece +1 energy for pieces color
      board_energy[i]=board_e_w[i] -board_e_b[i];
    }
    for (i = 98;i < 106; i++)
    { //if pieces on square defended by atleast one piece +1 energy for pieces color
      board_energy[i]=board_e_w[i] -board_e_b[i];
    }
    for (i = 110;i < 118; i++)
    { //if pieces on square defended by atleast one piece +1 energy for pieces color
      board_energy[i]=board_e_w[i] -board_e_b[i];
    }


 return;
}



